@@ -1,13 +1,19 @@
 import string
 import os
+import pathlib
 
 import vamb
 
 PARENTDIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 DATADIR = os.path.join(PARENTDIR, "test", "data")
-BAM_FILES = [
-    os.path.join(DATADIR, "bam", i) for i in os.listdir(os.path.join(DATADIR, "bam"))
-]
+BAM_FILES = sorted(
+    [
+        pathlib.Path(DATADIR).joinpath("bam").joinpath(i)
+        for i in os.listdir(os.path.join(DATADIR, "bam"))
+    ]
+)
+AEMB_DIR = os.path.join(DATADIR, "aemb")
+AEMB_FILES = sorted([pathlib.Path(AEMB_DIR).joinpath(i) for i in os.listdir(AEMB_DIR)])
 
 BAM_NAMES = [
     "S27C175628",
