@@ -1,10 +1,7 @@
 """Adversarial autoencoders (AAE) for metagenomics binning, this files contains the implementation of the AAE"""
 
-
 import numpy as np
-from math import log
-import time
-from torch.utils.data.dataset import TensorDataset as TensorDataset
+from math import log, isfinite
 from torch.autograd import Variable
 from torch.distributions.relaxed_categorical import RelaxedOneHotCategorical
 import torch.nn as nn
@@ -14,13 +11,9 @@ from vamb.encode import set_batchsize
 
 from collections.abc import Sequence
 from typing import Optional, IO, Union
+from numpy.typing import NDArray
+from loguru import logger
 
-random_seed = 42
-torch.manual_seed(random_seed)
-torch.cuda.manual_seed(random_seed)
-torch.backends.cudnn.deterministic = True
-torch.backends.cudnn.benchmark = False
-np.random.seed(random_seed)
 
 ############################################################################# MODEL ###########################################################
 class AAE(nn.Module):
@@ -34,11 +27,30 @@ class AAE(nn.Module):
         slr: float,
         alpha: Optional[float],
         _cuda: bool,
+        seed: int,
     ):
-        if nsamples is None:
-            raise ValueError(
-                f"Number of samples  should be provided to define the encoder input layer as well as the categorical latent dimension, not {nsamples}"
-            )
+        for variable, name in [
+            (nsamples, "nsamples"),
+            (nhiddens, "nhiddens"),
+            (nlatent_l, "nlatents_l"),
+            (nlatent_y, "nlatents_y"),
+        ]:
+            if variable < 1:
+                raise ValueError(f"{name} must be at least 1, not {variable}")
+
+        real_variables = [(sl, "sl"), (slr, "slr")]
+        if alpha is not None:
+            real_variables.append((alpha, "alpha"))
+        for variable, name in real_variables:
+            if not isfinite(variable) or not (0.0 <= variable <= 1.0):
+                raise ValueError(
+                    f"{name} must be in the interval [0.0, 1.0], not {variable}"
+                )
+
+        torch.manual_seed(seed)
+        torch.cuda.manual_seed(seed)
+        torch.backends.cudnn.deterministic = True
+        torch.backends.cudnn.benchmark = False
 
         super(AAE, self).__init__()
         if alpha is None:
@@ -54,6 +66,7 @@ class AAE(nn.Module):
         self.slr = slr
         self.alpha = alpha
         self.usecuda = _cuda
+        self.rng = np.random.Generator(np.random.PCG64(seed))
 
         # encoder
         self.encoder = nn.Sequential(
@@ -105,11 +118,10 @@ class AAE(nn.Module):
 
     ## Reparametrisation trick
     def _reparameterization(self, mu, logvar):
-
         Tensor = torch.cuda.FloatTensor if self.usecuda else torch.FloatTensor
 
         std = torch.exp(logvar / 2)
-        sampled_z = Variable(Tensor(np.random.normal(0, 1, (mu.size(0), self.ld))))
+        sampled_z = Variable(Tensor(self.rng.normal(0, 1, (mu.size(0), self.ld))))
 
         if self.usecuda:
             sampled_z = sampled_z.cuda()
@@ -194,32 +206,28 @@ class AAE(nn.Module):
         nepochs: int,
         batchsteps: list[int],
         T,
-        lr: float,
-        logfile: Optional[IO[str]] = None,
         modelfile: Union[None, str, IO[bytes]] = None,
     ):
-
+        lr = 0.001
         Tensor = torch.cuda.FloatTensor if self.usecuda else torch.FloatTensor
         batchsteps_set = set(batchsteps)
         ncontigs, _ = data_loader.dataset.tensors[0].shape
 
         # Initialize generator and discriminator
-
-        if logfile is not None:
-            print("\tNetwork properties:", file=logfile)
-            print("\tCUDA:", self.usecuda, file=logfile)
-            print("\tAlpha:", self.alpha, file=logfile)
-            print("\tY length:", self.y_len, file=logfile)
-            print("\tZ length:", self.ld, file=logfile)
-            print("\n\tTraining properties:", file=logfile)
-            print("\tN epochs:", nepochs, file=logfile)
-            print("\tStarting batch size:", data_loader.batch_size, file=logfile)
-            batchsteps_string = (
-                ", ".join(map(str, sorted(batchsteps))) if batchsteps_set else "None"
-            )
-            print("\tBatchsteps:", batchsteps_string, file=logfile)
-            print("\tN sequences:", ncontigs, file=logfile)
-            print("\tN samples:", self.nsamples, file=logfile, end="\n\n")
+        logger.info("\tNetwork properties:")
+        logger.info(f"\t    CUDA: {self.usecuda}")
+        logger.info(f"\t    Alpha: {self.alpha}")
+        logger.info(f"\t    Y length: {self.y_len}")
+        logger.info(f"\t    Z length: {self.ld}")
+        logger.info("\tTraining properties:")
+        logger.info(f"\t    N epochs: {nepochs}")
+        logger.info(f"\t    Starting batch size: {data_loader.batch_size}")
+        batchsteps_string = (
+            ", ".join(map(str, sorted(batchsteps))) if batchsteps_set else "None"
+        )
+        logger.info(f"\t    Batchsteps: {batchsteps_string}")
+        logger.info(f"\t    N sequences: {ncontigs}")
+        logger.info(f"\t    N samples: {self.nsamples}")
 
         # we need to separate the paramters due to the adversarial training
 
@@ -251,7 +259,11 @@ class AAE(nn.Module):
 
         for epoch_i in range(nepochs):
             if epoch_i in batchsteps:
-                data_loader = set_batchsize(data_loader, data_loader.batch_size * 2)
+                data_loader = set_batchsize(
+                    data_loader,
+                    data_loader.batch_size * 2,  # type:ignore
+                    data_loader.dataset.tensors[0].shape[0],
+                )
 
             (
                 ED_loss_e,
@@ -263,9 +275,9 @@ class AAE(nn.Module):
             ) = (0, 0, 0, 0, 0, 0)
 
             total_batches_inthis_epoch = len(data_loader)
-            time_epoch_0 = time.time()
 
-            for depths_in, tnfs_in, _ in data_loader:  # weights currently unused here
+            # weights, abundances currently unused here
+            for depths_in, tnfs_in, _, _ in data_loader:
                 nrows, _ = depths_in.shape
 
                 # Adversarial ground truths
@@ -290,7 +302,7 @@ class AAE(nn.Module):
                     y_prior = y_prior.cuda()
 
                 else:
-                    z_prior = Variable(Tensor(np.random.normal(0, 1, (nrows, self.ld))))
+                    z_prior = Variable(Tensor(self.rng.normal(0, 1, (nrows, self.ld))))
                     ohc = RelaxedOneHotCategorical(T, torch.ones([nrows, self.y_len]))
                     y_prior = ohc.sample()
 
@@ -380,25 +392,18 @@ class AAE(nn.Module):
                 CE_e += float(ce.item())
                 SSE_e += float(sse.item())
 
-            time_epoch_1 = time.time()
-            time_e = np.round((time_epoch_1 - time_epoch_0) / 60, 3)
-
-            if logfile is not None:
-                print(
-                    "\tEpoch: {}\t Loss Enc/Dec: {:.6f}\t Rec. loss: {:.4f}\t CE: {:.4f}\tSSE: {:.4f}\t Dz loss: {:.7f}\t Dy loss: {:.6f}\t Batchsize: {}\t Epoch time(min): {: .4}".format(
-                        epoch_i + 1,
-                        ED_loss_e / total_batches_inthis_epoch,
-                        V_loss_e / total_batches_inthis_epoch,
-                        CE_e / total_batches_inthis_epoch,
-                        SSE_e / total_batches_inthis_epoch,
-                        D_z_loss_e / total_batches_inthis_epoch,
-                        D_y_loss_e / total_batches_inthis_epoch,
-                        data_loader.batch_size,
-                        time_e,
-                    ),
-                    file=logfile,
-                )
-                logfile.flush()
+            logger.info(
+                "\t\tEpoch: {:>3} Loss Enc/Dec: {:.5e} Rec. loss: {:.5e} CE: {:.5e} SSE: {:.5e} Dz loss: {:.5e} Dy loss: {:.5e} Batchsize: {:>4}".format(
+                    epoch_i + 1,
+                    ED_loss_e / total_batches_inthis_epoch,
+                    V_loss_e / total_batches_inthis_epoch,
+                    CE_e / total_batches_inthis_epoch,
+                    SSE_e / total_batches_inthis_epoch,
+                    D_z_loss_e / total_batches_inthis_epoch,
+                    D_y_loss_e / total_batches_inthis_epoch,
+                    data_loader.batch_size,
+                ),
+            )
 
             # save model
             if modelfile is not None:
@@ -426,7 +431,9 @@ class AAE(nn.Module):
         return None
 
     ########### funciton that retrieves the clusters from Y latents
-    def get_latents(self, contignames: Sequence[str], data_loader, last_epoch: bool = True):
+    def get_latents(
+        self, contignames: Sequence[str], data_loader
+    ) -> tuple[dict[str, set[str]], NDArray[np.float32]]:
         """Retrieve the categorical latent representation (y) and the contiouous latents (l) of the inputs
 
         Inputs:
@@ -440,18 +447,19 @@ class AAE(nn.Module):
             l_latents array"""
         self.eval()
 
-        new_data_loader = set_batchsize(data_loader, 256, encode=True)
-        depths_array, _, _ = data_loader.dataset.tensors
+        depths_array, _, _, _ = data_loader.dataset.tensors
+        new_data_loader = set_batchsize(
+            data_loader, 256, len(depths_array), encode=True
+        )
 
         length = len(depths_array)
         latent = np.empty((length, self.ld), dtype=np.float32)
         index_contigname = 0
         row = 0
-        clust_y_dict = dict()
+        clust_y_dict: dict[str, set[str]] = dict()
         Tensor = torch.cuda.FloatTensor if self.usecuda else torch.FloatTensor
         with torch.no_grad():
-
-            for depths_in, tnfs_in, _ in new_data_loader:
+            for depths_in, tnfs_in, _, _ in new_data_loader:
                 nrows, _ = depths_in.shape
                 if self.usecuda:
                     z_prior = torch.cuda.FloatTensor(nrows, self.ld).normal_()
@@ -464,7 +472,7 @@ class AAE(nn.Module):
                     y_prior = y_prior.cuda()
 
                 else:
-                    z_prior = Variable(Tensor(np.random.normal(0, 1, (nrows, self.ld))))
+                    z_prior = Variable(Tensor(self.rng.normal(0, 1, (nrows, self.ld))))
                     ohc = RelaxedOneHotCategorical(
                         0.15, torch.ones([nrows, self.y_len])
                     )
@@ -474,25 +482,18 @@ class AAE(nn.Module):
                     depths_in = depths_in.cuda()
                     tnfs_in = tnfs_in.cuda()
 
-                if last_epoch:
-                    mu, _, _, _, y_sample = self(depths_in, tnfs_in)[
-                        0:5
-                    ]
-                else:
-                    y_sample = self(depths_in, tnfs_in)[4]
+                mu, _, _, _, y_sample = self(depths_in, tnfs_in)[0:5]
 
                 if self.usecuda:
                     Ys = y_sample.cpu().detach().numpy()
-                    if last_epoch:
-                        mu = mu.cpu().detach().numpy()
-                        latent[row : row + len(mu)] = mu
-                        row += len(mu)
+                    mu = mu.cpu().detach().numpy()
+                    latent[row : row + len(mu)] = mu
+                    row += len(mu)
                 else:
                     Ys = y_sample.detach().numpy()
-                    if last_epoch:
-                        mu = mu.detach().numpy()
-                        latent[row : row + len(mu)] = mu
-                        row += len(mu)
+                    mu = mu.detach().numpy()
+                    latent[row : row + len(mu)] = mu
+                    row += len(mu)
                 del y_sample
 
                 for _y in Ys:
@@ -508,7 +509,4 @@ class AAE(nn.Module):
                     index_contigname += 1
                 del Ys
 
-            if last_epoch:
-                return clust_y_dict, latent
-            else:
-                return clust_y_dict
+            return clust_y_dict, latent
