@@ -4,10 +4,11 @@ import vamb.vambtools as _vambtools
 from torch.utils.data.dataset import TensorDataset as _TensorDataset
 from torch.utils.data import DataLoader as _DataLoader
 from torch.nn.functional import softmax as _softmax
-from torch.optim import Adam as _Adam
 from torch import Tensor
+import dadaptation
 from torch import nn as _nn
 from math import log as _log
+from loguru import logger
 
 __doc__ = """Encode a depths matrix and a tnf matrix to latent representation.
 
@@ -16,7 +17,7 @@ and tnf in the latent space under gaussian noise.
 
 Usage:
 >>> vae = VAE(nsamples=6)
->>> dataloader, mask = make_dataloader(depths, tnf, lengths)
+>>> dataloader = make_dataloader(depths, tnf, lengths)
 >>> vae.trainmodel(dataloader)
 >>> latent = vae.encode(dataloader) # Encode to latent representation
 >>> latent.shape
@@ -28,9 +29,10 @@ __cmd_doc__ = """Encode depths and TNF using a VAE to latent representation"""
 import numpy as _np
 import torch as _torch
 
-_torch.manual_seed(0)
 
-def set_batchsize(data_loader: _DataLoader, batch_size: int, encode=False) -> _DataLoader:
+def set_batchsize(
+    data_loader: _DataLoader, batch_size: int, n_obs: int, encode=False
+) -> _DataLoader:
     """Effectively copy the data loader, but with a different batch size.
 
     The `encode` option is used to copy the dataloader to use before encoding.
@@ -39,121 +41,108 @@ def set_batchsize(data_loader: _DataLoader, batch_size: int, encode=False) -> _D
     """
     return _DataLoader(
         dataset=data_loader.dataset,
-        batch_size= batch_size,
+        batch_size=batch_size,
         shuffle=not encode,
-        drop_last=not encode,
+        drop_last=not encode and (n_obs > batch_size),
         num_workers=1 if encode else data_loader.num_workers,
         pin_memory=data_loader.pin_memory,
     )
 
+
 def make_dataloader(
-    rpkm: _np.ndarray,
+    abundance: _np.ndarray,
     tnf: _np.ndarray,
     lengths: _np.ndarray,
     batchsize: int = 256,
     destroy: bool = False,
     cuda: bool = False,
-) -> tuple[_DataLoader[tuple[Tensor, Tensor, Tensor]], _np.ndarray]:
-    """Create a DataLoader and a contig mask from RPKM and TNF.
+) -> _DataLoader:
+    """Create a DataLoader from abundance, TNF and lengths.
 
     The dataloader is an object feeding minibatches of contigs to the VAE.
-    The data are normalized versions of the input datasets, with zero-contigs,
-    i.e. contigs where a row in either TNF or RPKM are all zeros, removed.
-    The mask is a boolean mask designating which contigs have been kept.
+    The data are normalized versions of the input datasets.
 
     Inputs:
-        rpkm: RPKM matrix (N_contigs x N_samples)
+        abundance: Abundance matrix (N_contigs x N_samples)
         tnf: TNF matrix (N_contigs x N_TNF)
+        lengths: Numpy array of sequence length (N_contigs)
         batchsize: Starting size of minibatches for dataloader
-        destroy: Mutate rpkm and tnf array in-place instead of making a copy.
+        destroy: Mutate abundance and tnf array in-place instead of making a copy.
         cuda: Pagelock memory of dataloader (use when using GPU acceleration)
 
     Outputs:
         DataLoader: An object feeding data to the VAE
-        mask: A boolean mask of which contigs are kept
     """
 
-    if not isinstance(rpkm, _np.ndarray) or not isinstance(tnf, _np.ndarray):
-        raise ValueError("TNF and RPKM must be Numpy arrays")
+    if not isinstance(abundance, _np.ndarray) or not isinstance(tnf, _np.ndarray):
+        raise ValueError("TNF and abundance must be Numpy arrays")
 
     if batchsize < 1:
         raise ValueError(f"Batch size must be minimum 1, not {batchsize}")
 
-    if len(rpkm) != len(tnf) or len(tnf) != len(lengths):
-        raise ValueError("Lengths of RPKM, TNF and lengths arrays must be the same")
+    if len(abundance) != len(tnf) or len(tnf) != len(lengths):
+        raise ValueError(
+            "Lengths of abundance, TNF and lengths arrays must be the same"
+        )
 
-    if not (rpkm.dtype == tnf.dtype == _np.float32):
-        raise ValueError("TNF and RPKM must be Numpy arrays of dtype float32")
+    if not (abundance.dtype == tnf.dtype == _np.float32):
+        raise ValueError("TNF and abundance must be Numpy arrays of dtype float32")
 
-    ### Copy arrays and mask them ###
     # Copy if not destroy - this way we can have all following operations in-place
     # for simplicity
     if not destroy:
-        rpkm = rpkm.copy()
+        abundance = abundance.copy()
         tnf = tnf.copy()
 
     # Normalize samples to have same depth
-    sample_depths_sum = rpkm.sum(axis=0)
+    sample_depths_sum = abundance.sum(axis=0)
     if _np.any(sample_depths_sum == 0):
         raise ValueError(
             "One or more samples have zero depth in all sequences, so cannot be depth normalized"
         )
-    rpkm *= 1_000_000 / sample_depths_sum
+    abundance *= 1_000_000 / sample_depths_sum
+    total_abundance = abundance.sum(axis=1)
 
-    # If multiple samples, also include nonzero depth as requirement for accept
-    # of sequences
-    mask = tnf.sum(axis=1) != 0
-    depthssum = None
-    if rpkm.shape[1] > 1:
-        depthssum = rpkm.sum(axis=1)
-        mask &= depthssum != 0
-        depthssum = depthssum[mask]
-        assert isinstance(depthssum, _np.ndarray)
+    # Normalize abundance to sum to 1
+    n_samples = abundance.shape[1]
+    zero_total_abundance = total_abundance == 0
+    abundance[zero_total_abundance] = 1 / n_samples
+    nonzero_total_abundance = total_abundance.copy()
+    nonzero_total_abundance[zero_total_abundance] = 1.0
+    abundance /= nonzero_total_abundance.reshape((-1, 1))
 
-    if mask.sum() < batchsize:
-        raise ValueError(
-            "Fewer sequences left after filtering than the batch size. "
-            + "This probably means you try to run on a too small dataset (below ~10k sequences), "
-            + "or that nearly all sequences were filtered away. Check the log file, "
-            + "and verify BAM file content is sensible."
-        )
-
-    _vambtools.numpy_inplace_maskarray(rpkm, mask)
-    _vambtools.numpy_inplace_maskarray(tnf, mask)
-
-    # If multiple samples, normalize to sum to 1, else zscore normalize
-    if rpkm.shape[1] > 1:
-        assert depthssum is not None  # we set it so just above
-        rpkm /= depthssum.reshape((-1, 1))
-    else:
-        _vambtools.zscore(rpkm, axis=0, inplace=True)
-
-    # Normalize TNF
+    # Normalize TNF and total abundance to make SSE loss work better
+    total_abundance = _np.log(total_abundance.clip(min=0.001))
+    _vambtools.zscore(total_abundance, inplace=True)
     _vambtools.zscore(tnf, axis=0, inplace=True)
+    total_abundance.shape = (len(total_abundance), 1)
 
     # Create weights
-    lengths = (lengths[mask]).astype(_np.float32)
+    lengths = (lengths).astype(_np.float32)
     weights = _np.log(lengths).astype(_np.float32) - 5.0
     weights[weights < 2.0] = 2.0
     weights *= len(weights) / weights.sum()
     weights.shape = (len(weights), 1)
 
     ### Create final tensors and dataloader ###
-    depthstensor = _torch.from_numpy(rpkm)  # this is a no-copy operation
+    depthstensor = _torch.from_numpy(abundance)  # this is a no-copy operation
     tnftensor = _torch.from_numpy(tnf)
+    total_abundance_tensor = _torch.from_numpy(total_abundance)
     weightstensor = _torch.from_numpy(weights)
     n_workers = 4 if cuda else 1
-    dataset = _TensorDataset(depthstensor, tnftensor, weightstensor)
+    dataset = _TensorDataset(
+        depthstensor, tnftensor, total_abundance_tensor, weightstensor
+    )
     dataloader = _DataLoader(
         dataset=dataset,
         batch_size=batchsize,
-        drop_last=True,
+        drop_last=(len(depthstensor) > batchsize),
         shuffle=True,
         num_workers=n_workers,
         pin_memory=cuda,
     )
 
-    return dataloader, mask
+    return dataloader
 
 
 class VAE(_nn.Module):
@@ -168,7 +157,7 @@ class VAE(_nn.Module):
         dropout: Probability of dropout on forward pass [0.2]
         cuda: Use CUDA (GPU accelerated training) [False]
 
-    vae.trainmodel(dataloader, nepochs batchsteps, lrate, logfile, modelfile)
+    vae.trainmodel(dataloader, nepochs batchsteps, modelfile)
         Trains the model, returning None
 
     vae.encode(self, data_loader):
@@ -187,6 +176,7 @@ class VAE(_nn.Module):
         beta: float = 200.0,
         dropout: Optional[float] = 0.2,
         cuda: bool = False,
+        seed: int = 0,
     ):
         if nlatent < 1:
             raise ValueError(f"Minimum 1 latent neuron, not {nlatent}")
@@ -216,6 +206,9 @@ class VAE(_nn.Module):
         if not (0 <= dropout < 1):
             raise ValueError(f"dropout must be 0 <= dropout < 1, not {dropout}")
 
+        _torch.manual_seed(seed)
+        self.rng = _torch.Generator()
+        self.rng.manual_seed(seed)
         super(VAE, self).__init__()
 
         # Initialize simple attributes
@@ -236,22 +229,23 @@ class VAE(_nn.Module):
 
         # Add all other hidden layers
         for nin, nout in zip(
-            [self.nsamples + self.ntnf] + self.nhiddens, self.nhiddens
+            # + 1 for the total abundance
+            [self.nsamples + self.ntnf + 1] + self.nhiddens,
+            self.nhiddens,
         ):
             self.encoderlayers.append(_nn.Linear(nin, nout))
             self.encodernorms.append(_nn.BatchNorm1d(nout))
 
         # Latent layers
         self.mu = _nn.Linear(self.nhiddens[-1], self.nlatent)
-        self.logsigma = _nn.Linear(self.nhiddens[-1], self.nlatent)
 
         # Add first decoding layer
         for nin, nout in zip([self.nlatent] + self.nhiddens[::-1], self.nhiddens[::-1]):
             self.decoderlayers.append(_nn.Linear(nin, nout))
             self.decodernorms.append(_nn.BatchNorm1d(nout))
 
-        # Reconstruction (output) layer
-        self.outputlayer = _nn.Linear(self.nhiddens[0], self.nsamples + self.ntnf)
+        # Reconstruction (output) layer. + 1 for the total abundance
+        self.outputlayer = _nn.Linear(self.nhiddens[0], self.nsamples + self.ntnf + 1)
 
         # Activation functions
         self.relu = _nn.LeakyReLU()
@@ -261,7 +255,7 @@ class VAE(_nn.Module):
         if cuda:
             self.cuda()
 
-    def _encode(self, tensor: Tensor) -> tuple[Tensor, Tensor]:
+    def _encode(self, tensor: Tensor) -> Tensor:
         tensors = list()
 
         # Hidden layers
@@ -272,21 +266,13 @@ class VAE(_nn.Module):
         # Latent layers
         mu = self.mu(tensor)
 
-        # Note: This softplus constrains logsigma to positive. As reconstruction loss pushes
-        # logsigma as low as possible, and KLD pushes it towards 0, the optimizer will
-        # always push this to 0, meaning that the logsigma layer will be pushed towards
-        # negative infinity. This creates a nasty numerical instability in VAMB. Luckily,
-        # the gradient also disappears as it decreases towards negative infinity, avoiding
-        # NaN poisoning in most cases. We tried to remove the softplus layer, but this
-        # necessitates a new round of hyperparameter optimization, and there is no way in
-        # hell I am going to do that at the moment of writing.
-        # Also remove needless factor 2 in definition of latent in reparameterize function.
-        logsigma = self.softplus(self.logsigma(tensor))
+        # Note: We ought to also compute logsigma here, but we had a bug in the original
+        # implementation of Vamb where logsigma was fixed to zero, so we just remove it.
 
-        return mu, logsigma
+        return mu
 
     # sample with gaussian noise
-    def reparameterize(self, mu: Tensor, logsigma: Tensor) -> Tensor:
+    def reparameterize(self, mu: Tensor) -> Tensor:
         epsilon = _torch.randn(mu.size(0), mu.size(1))
 
         if self.usecuda:
@@ -294,12 +280,11 @@ class VAE(_nn.Module):
 
         epsilon.requires_grad = True
 
-        # See comment above regarding softplus
-        latent = mu + epsilon * _torch.exp(logsigma / 2)
+        latent = mu + epsilon
 
         return latent
 
-    def _decode(self, tensor: Tensor) -> tuple[Tensor, Tensor]:
+    def _decode(self, tensor: Tensor) -> tuple[Tensor, Tensor, Tensor]:
         tensors = list()
 
         for decoderlayer, decodernorm in zip(self.decoderlayers, self.decodernorms):
@@ -311,22 +296,21 @@ class VAE(_nn.Module):
         # Decompose reconstruction to depths and tnf signal
         depths_out = reconstruction.narrow(1, 0, self.nsamples)
         tnf_out = reconstruction.narrow(1, self.nsamples, self.ntnf)
+        abundance_out = reconstruction.narrow(1, self.nsamples + self.ntnf, 1)
 
-        # If multiple samples, apply softmax
-        if self.nsamples > 1:
-            depths_out = _softmax(depths_out, dim=1)
+        depths_out = _softmax(depths_out, dim=1)
 
-        return depths_out, tnf_out
+        return depths_out, tnf_out, abundance_out
 
     def forward(
-        self, depths: Tensor, tnf: Tensor
+        self, depths: Tensor, tnf: Tensor, abundance: Tensor
     ) -> tuple[Tensor, Tensor, Tensor, Tensor]:
-        tensor = _torch.cat((depths, tnf), 1)
-        mu, logsigma = self._encode(tensor)
-        latent = self.reparameterize(mu, logsigma)
-        depths_out, tnf_out = self._decode(latent)
+        tensor = _torch.cat((depths, tnf, abundance), 1)
+        mu = self._encode(tensor)
+        latent = self.reparameterize(mu)
+        depths_out, tnf_out, abundance_out = self._decode(latent)
 
-        return depths_out, tnf_out, mu, logsigma
+        return depths_out, tnf_out, abundance_out, mu
 
     def calc_loss(
         self,
@@ -334,28 +318,42 @@ class VAE(_nn.Module):
         depths_out: Tensor,
         tnf_in: Tensor,
         tnf_out: Tensor,
+        abundance_in: Tensor,
+        abundance_out: Tensor,
         mu: Tensor,
-        logsigma: Tensor,
         weights: Tensor,
-    ) -> tuple[Tensor, Tensor, Tensor, Tensor]:
-        # If multiple samples, use cross entropy, else use SSE for abundance
-        if self.nsamples > 1:
-            # Add 1e-9 to depths_out to avoid numerical instability.
-            ce = -((depths_out + 1e-9).log() * depths_in).sum(dim=1)
-            ce_weight = (1 - self.alpha) / _log(self.nsamples)
-        else:
-            ce = (depths_out - depths_in).pow(2).sum(dim=1)
-            ce_weight = 1 - self.alpha
-
+    ) -> tuple[Tensor, Tensor, Tensor, Tensor, Tensor]:
+        ab_sse = (abundance_out - abundance_in).pow(2).sum(dim=1)
+        # Add 1e-9 to depths_out to avoid numerical instability.
+        ce = -((depths_out + 1e-9).log() * depths_in).sum(dim=1)
         sse = (tnf_out - tnf_in).pow(2).sum(dim=1)
-        kld = -0.5 * (1 + logsigma - mu.pow(2) - logsigma.exp()).sum(dim=1)
+        kld = 0.5 * (mu.pow(2)).sum(dim=1)
+
+        # Avoid having the denominator be zero
+        if self.nsamples == 1:
+            ce_weight = 0.0
+        else:
+            ce_weight = ((1 - self.alpha) * (self.nsamples - 1)) / (
+                self.nsamples * _log(self.nsamples)
+            )
+
+        ab_sse_weight = (1 - self.alpha) * (1 / self.nsamples)
         sse_weight = self.alpha / self.ntnf
         kld_weight = 1 / (self.nlatent * self.beta)
-        reconstruction_loss = ce * ce_weight + sse * sse_weight
-        kld_loss = kld * kld_weight
-        loss = (reconstruction_loss + kld_loss) * weights
+        weighed_ab = ab_sse * ab_sse_weight
+        weighed_ce = ce * ce_weight
+        weighed_sse = sse * sse_weight
+        weighed_kld = kld * kld_weight
+        reconstruction_loss = weighed_ce + weighed_ab + weighed_sse
+        loss = (reconstruction_loss + weighed_kld) * weights
 
-        return loss.mean(), ce.mean(), sse.mean(), kld.mean()
+        return (
+            loss.mean(),
+            weighed_ab.mean(),
+            weighed_ce.mean(),
+            weighed_sse.mean(),
+            weighed_kld.mean(),
+        )
 
     def trainepoch(
         self,
@@ -363,33 +361,57 @@ class VAE(_nn.Module):
         epoch: int,
         optimizer,
         batchsteps: list[int],
-        logfile,
     ) -> _DataLoader[tuple[Tensor, Tensor, Tensor]]:
+        n_seq = len(data_loader.dataset.tensors[0])
+        if n_seq < 2:
+            raise ValueError(
+                "Cannot train on a dataset with fewer than 2 sequences, but got "
+                f"{n_seq} sequences. "
+                "If you are trying to fit a DL model to this few sequences, "
+                "something probably went wrong in your pipeline."
+            )
+
         self.train()
 
         epoch_loss = 0.0
         epoch_kldloss = 0.0
         epoch_sseloss = 0.0
         epoch_celoss = 0.0
+        epoch_absseloss = 0.0
 
         if epoch in batchsteps:
-            data_loader =  set_batchsize(data_loader, data_loader.batch_size * 2)
+            data_loader = set_batchsize(
+                data_loader,
+                data_loader.batch_size * 2,  # type:ignore
+                n_seq,
+            )
 
-        for depths_in, tnf_in, weights in data_loader:
+        for depths_in, tnf_in, abundance_in, weights in data_loader:
             depths_in.requires_grad = True
             tnf_in.requires_grad = True
+            abundance_in.requires_grad = True
 
             if self.usecuda:
                 depths_in = depths_in.cuda()
                 tnf_in = tnf_in.cuda()
+                abundance_in = abundance_in.cuda()
                 weights = weights.cuda()
 
             optimizer.zero_grad()
 
-            depths_out, tnf_out, mu, logsigma = self(depths_in, tnf_in)
+            depths_out, tnf_out, abundance_out, mu = self(
+                depths_in, tnf_in, abundance_in
+            )
 
-            loss, ce, sse, kld = self.calc_loss(
-                depths_in, depths_out, tnf_in, tnf_out, mu, logsigma, weights
+            loss, ab_sse, ce, sse, kld = self.calc_loss(
+                depths_in,
+                depths_out,
+                tnf_in,
+                tnf_out,
+                abundance_in,
+                abundance_out,
+                mu,
+                weights,
             )
 
             loss.backward()
@@ -399,21 +421,19 @@ class VAE(_nn.Module):
             epoch_kldloss += kld.data.item()
             epoch_sseloss += sse.data.item()
             epoch_celoss += ce.data.item()
+            epoch_absseloss += ab_sse.data.item()
 
-        if logfile is not None:
-            print(
-                "\tEpoch: {}\tLoss: {:.6f}\tCE: {:.7f}\tSSE: {:.6f}\tKLD: {:.4f}\tBatchsize: {}".format(
-                    epoch + 1,
-                    epoch_loss / len(data_loader),
-                    epoch_celoss / len(data_loader),
-                    epoch_sseloss / len(data_loader),
-                    epoch_kldloss / len(data_loader),
-                    data_loader.batch_size,
-                ),
-                file=logfile,
+        logger.info(
+            "\t\tEpoch: {:>3}  Loss: {:.5e}  CE: {:.5e}  AB: {:.5e}  SSE: {:.5e}  KLD: {:.5e}  Batchsize: {:>4}".format(
+                epoch + 1,
+                epoch_loss / len(data_loader),
+                epoch_celoss / len(data_loader),
+                epoch_absseloss / len(data_loader),
+                epoch_sseloss / len(data_loader),
+                epoch_kldloss / len(data_loader),
+                data_loader.batch_size,
             )
-
-            logfile.flush()
+        )
 
         self.eval()
         return data_loader
@@ -428,9 +448,11 @@ class VAE(_nn.Module):
 
         self.eval()
 
-        new_data_loader = set_batchsize(data_loader, data_loader.batch_size, encode=True)
+        depths_array, _, _, _ = data_loader.dataset.tensors
+        new_data_loader = set_batchsize(
+            data_loader, data_loader.batch_size, len(depths_array), encode=True
+        )
 
-        depths_array, _, _ = data_loader.dataset.tensors
         length = len(depths_array)
 
         # We make a Numpy array instead of a Torch array because, if we create
@@ -440,14 +462,15 @@ class VAE(_nn.Module):
 
         row = 0
         with _torch.no_grad():
-            for depths, tnf, weights in new_data_loader:
+            for depths, tnf, ab, _ in new_data_loader:
                 # Move input to GPU if requested
                 if self.usecuda:
                     depths = depths.cuda()
                     tnf = tnf.cuda()
+                    ab = ab.cuda()
 
                 # Evaluate
-                _, _, mu, _ = self(depths, tnf)
+                _, _, _, mu = self(depths, tnf, ab)
 
                 if self.usecuda:
                     mu = mu.cpu()
@@ -456,6 +479,7 @@ class VAE(_nn.Module):
                 row += len(mu)
 
         assert row == length
+        _vambtools.mask_lower_bits(latent, 12)
         return latent
 
     def save(self, filehandle):
@@ -492,7 +516,9 @@ class VAE(_nn.Module):
         """
 
         # Forcably load to CPU even if model was saves as GPU model
-        dictionary = _torch.load(path, map_location=lambda storage, loc: storage)
+        dictionary = _torch.load(
+            path, map_location=lambda storage, loc: storage, weights_only=True
+        )
 
         nsamples = dictionary["nsamples"]
         alpha = dictionary["alpha"]
@@ -517,9 +543,7 @@ class VAE(_nn.Module):
         self,
         dataloader: _DataLoader[tuple[Tensor, Tensor, Tensor]],
         nepochs: int = 500,
-        lrate: float = 1e-3,
         batchsteps: Optional[list[int]] = [25, 75, 150, 300],
-        logfile: Optional[IO[str]] = None,
         modelfile: Union[None, str, Path, IO[bytes]] = None,
     ):
         """Train the autoencoder from depths array and tnf array.
@@ -527,17 +551,11 @@ class VAE(_nn.Module):
         Inputs:
             dataloader: DataLoader made by make_dataloader
             nepochs: Train for this many epochs before encoding [500]
-            lrate: Starting learning rate for the optimizer [0.001]
             batchsteps: None or double batchsize at these epochs [25, 75, 150, 300]
-            logfile: Print status updates to this file if not None [None]
             modelfile: Save models to this file if not None [None]
 
         Output: None
         """
-
-        if lrate < 0:
-            raise ValueError(f"Learning rate must be positive, not {lrate}")
-
         if nepochs < 1:
             raise ValueError("Minimum 1 epoch, not {nepochs}")
 
@@ -551,49 +569,34 @@ class VAE(_nn.Module):
                 raise ValueError("All elements of batchsteps must be integers")
             if max(batchsteps, default=0) >= nepochs:
                 raise ValueError("Max batchsteps must not equal or exceed nepochs")
-            last_batchsize = dataloader.batch_size * 2 ** len(batchsteps)
-            if len(dataloader.dataset) < last_batchsize:  # type: ignore
-                raise ValueError(
-                    f"Last batch size of {last_batchsize} exceeds dataset length "
-                    f"of {len(dataloader.dataset)}. "  # type: ignore
-                    "This means you have too few contigs left after filtering to train. "
-                    "It is not adviced to run Vamb with fewer than 10,000 sequences "
-                    "after filtering. "
-                    "Please check the Vamb log file to see where the sequences were "
-                    "filtered away, and verify BAM files has sensible content."
-                )
             batchsteps_set = set(batchsteps)
 
         # Get number of features
         # Following line is un-inferrable due to typing problems with DataLoader
         ncontigs, nsamples = dataloader.dataset.tensors[0].shape  # type: ignore
-        optimizer = _Adam(self.parameters(), lr=lrate)
+        optimizer = dadaptation.DAdaptAdam(self.parameters(), decouple=True)
 
-        if logfile is not None:
-            print("\tNetwork properties:", file=logfile)
-            print("\tCUDA:", self.usecuda, file=logfile)
-            print("\tAlpha:", self.alpha, file=logfile)
-            print("\tBeta:", self.beta, file=logfile)
-            print("\tDropout:", self.dropout, file=logfile)
-            print("\tN hidden:", ", ".join(map(str, self.nhiddens)), file=logfile)
-            print("\tN latent:", self.nlatent, file=logfile)
-            print("\n\tTraining properties:", file=logfile)
-            print("\tN epochs:", nepochs, file=logfile)
-            print("\tStarting batch size:", dataloader.batch_size, file=logfile)
-            batchsteps_string = (
-                ", ".join(map(str, sorted(batchsteps_set)))
-                if batchsteps_set
-                else "None"
-            )
-            print("\tBatchsteps:", batchsteps_string, file=logfile)
-            print("\tLearning rate:", lrate, file=logfile)
-            print("\tN sequences:", ncontigs, file=logfile)
-            print("\tN samples:", nsamples, file=logfile, end="\n\n")
+        logger.info("\tNetwork properties:")
+        logger.info(f"\t    CUDA: {self.usecuda}")
+        logger.info(f"\t    Alpha: {self.alpha}")
+        logger.info(f"\t    Beta: {self.beta}")
+        logger.info(f"\t    Dropout: {self.dropout}")
+        logger.info(f"\t    N hidden: {', '.join(map(str, self.nhiddens))}")
+        logger.info(f"\t    N latent: {self.nlatent}")
+        logger.info("\tTraining properties:")
+        logger.info(f"\t    N epochs: {nepochs}")
+        logger.info(f"\t    Starting batch size: {dataloader.batch_size}")
+        batchsteps_string = (
+            ", ".join(map(str, sorted(batchsteps_set))) if batchsteps_set else "None"
+        )
+        logger.info(f"\t    Batchsteps: {batchsteps_string}")
+        logger.info(f"\t    N sequences: {ncontigs}")
+        logger.info(f"\t    N samples: {nsamples}")
 
         # Train
         for epoch in range(nepochs):
             dataloader = self.trainepoch(
-                dataloader, epoch, optimizer, sorted(batchsteps_set), logfile
+                dataloader, epoch, optimizer, sorted(batchsteps_set)
             )
 
         # Save weights - Lord forgive me, for I have sinned when catching all exceptions
