@@ -1,15 +1,8 @@
-__doc__ = """Calculate tetranucleotide frequency from a FASTA file.
-
-Usage:
->>> with open('/path/to/contigs.fna') as filehandle
-...     tnfs, contignames, lengths = read_contigs(filehandle)
-"""
-
 import os as _os
 import numpy as _np
 import vamb.vambtools as _vambtools
 from collections.abc import Iterable, Sequence
-from typing import IO, Union, TypeVar
+from typing import IO, Union, TypeVar, Optional
 from pathlib import Path
 
 # This kernel is created in src/create_kernel.py. See that file for explanation
@@ -55,7 +48,7 @@ class CompositionMetaData:
         self.lengths = lengths
         self.mask = mask
         self.minlength = minlength
-        self.refhash = _vambtools.hash_refnames(identifiers)
+        self.refhash = _vambtools.RefHasher.hash_refnames(identifiers)
 
     @property
     def nseqs(self) -> int:
@@ -73,14 +66,14 @@ class CompositionMetaData:
 
         self.identifiers = self.identifiers[mask]
         self.lengths = self.lengths[mask]
-        self.refhash = _vambtools.hash_refnames(self.identifiers)
+        self.refhash = _vambtools.RefHasher.hash_refnames(self.identifiers)
 
     def filter_min_length(self, length: int):
         "Set or reset minlength of this object"
         if length <= self.minlength:
             return None
 
-        self.filter_mask(self.lengths >= length)
+        self.filter_mask(self.lengths >= length)  # type:ignore
         self.minlength = length
 
 
@@ -121,7 +114,7 @@ class Composition:
         )
 
     @classmethod
-    def load(cls, io: Union[str, IO[bytes]]):
+    def load(cls, io: Union[str, IO[bytes], Path]):
         arrs = _np.load(io, allow_pickle=True)
         metadata = CompositionMetaData(
             _vambtools.validate_input_array(arrs["identifiers"]),
@@ -158,12 +151,17 @@ class Composition:
         raw.clear()
 
     @classmethod
-    def from_file(cls: type[C], filehandle: Iterable[bytes], minlength: int = 100) -> C:
+    def from_file(
+        cls: type[C],
+        filehandle: Iterable[bytes],
+        filename: Optional[str],
+        minlength: int = 2000,
+    ) -> C:
         """Parses a FASTA file open in binary reading mode, returning Composition.
 
         Input:
             filehandle: Filehandle open in binary mode of a FASTA file
-            minlength: Ignore any references shorter than N bases [100]
+            minlength: Ignore any references shorter than N bases [2000]
         """
 
         if minlength < 4:
@@ -174,27 +172,36 @@ class Composition:
         lengths = _vambtools.PushArray(_np.int32)
         mask = bytearray()  # we convert to Numpy at end
         contignames: list[str] = list()
-
-        entries = _vambtools.byte_iterfasta(filehandle)
+        entries = _vambtools.byte_iterfasta(filehandle, filename)
 
         for entry in entries:
-            skip = len(entry) < minlength
+            length = len(entry)
+            skip = length < minlength
             mask.append(not skip)
 
             if skip:
                 continue
 
-            raw.extend(entry.kmercounts(4))
+            counts = entry.kmercounts()
+            if counts.sum() == 0:
+                raise ValueError(
+                    f'TNF value of contig "{entry.header}" is all zeros. '
+                    + "This implies that the sequence contained no 4-mers of A, C, G, T or U, "
+                    + "making this sequence uninformative. This is probably a mistake. "
+                    + "Verify that the sequence contains usable information (e.g. is not all N's)"
+                )
+            raw.extend(counts)
 
             if len(raw) > 256000:
                 Composition._convert(raw, projected)
 
             lengths.append(len(entry))
-            contignames.append(entry.header)
+            contignames.append(entry.identifier)
 
         # Convert rest of contigs
         Composition._convert(raw, projected)
         tnfs_arr = projected.take()
+        _vambtools.mask_lower_bits(tnfs_arr, 12)
 
         # Don't use reshape since it creates a new array object with shared memory
         tnfs_arr.shape = (len(tnfs_arr) // 103, 103)
